@@ -21,13 +21,7 @@
 
 #include <goffice/goffice-config.h>
 #include <goffice/graph/gog-renderer-impl.h>
-#ifdef GOFFICE_WITH_GTK
-#include <goffice/graph/gog-renderer-pixbuf.h>
-#endif
-#ifdef GOFFICE_WITH_CAIRO
 #include <goffice/graph/gog-renderer-cairo.h>
-#endif
-#include <goffice/graph/gog-renderer-svg.h>
 #include <goffice/graph/gog-style.h>
 #include <goffice/graph/gog-graph.h>
 #include <goffice/graph/gog-graph-impl.h>
@@ -35,6 +29,9 @@
 #include <goffice/utils/go-units.h>
 #include <goffice/utils/go-font.h>
 #include <goffice/utils/go-math.h>
+#ifndef GOG_RENDERER_CAIRO_WITH_SVG
+#include <goffice/graph/gog-renderer-svg.h>
+#endif
 
 #include <gsf/gsf.h>
 #include <gsf/gsf-impl-utils.h>
@@ -322,15 +319,32 @@ gog_renderer_get_ring_wedge_bpath (double cx, double cy,
 				   double th0, double th1)
 {
 	ArtBpath *path;
-	double th_arc, th_out, th_in, th_delta, t;
+	double th_arc, th_out, th_in, th_delta, t, r;
 	int i, n_segs;
-	gboolean fill = rx_in >= 0.0 && ry_in >= 0.0;
+	gboolean fill;
 	gboolean draw_in, ellipse = FALSE;
 
+	if (rx_out < rx_in) {
+		r = rx_out;
+		rx_out = rx_in;
+		rx_in = r;
+	}
+	if (ry_out < ry_in) {
+		r = ry_out;
+		ry_out = ry_in;
+		ry_in = r;
+	}
+	/* Here we tolerate slightly negative values for inside radius
+	 * when deciding to fill. We use outside radius for comparison. */
+	fill = rx_in >= -(rx_out * 1e-6) && ry_in >= -(ry_out * 1e-6);
+
  	if (rx_out <= 0.0 || ry_out <= 0.0 || rx_out < rx_in || ry_out < ry_in)
 		return NULL;
 
-	draw_in = fill && (rx_in > rx_out / 1E6) && (ry_in > ry_out / 1E6);
+	/* Here also we use outside radius for comparison. If inside radius
+	 * is high enough, we'll emit an arc, otherwise we just use lines to
+	 * wedge center. */
+	draw_in = fill && (rx_in > rx_out * 1e-6) && (ry_in > ry_out * 1e-6);
 
 	if (th1 < th0) { t = th1; th1 = th0; th0 = t; }
 	if (go_add_epsilon (th1 - th0) >= 2 * M_PI) {
@@ -367,7 +381,15 @@ gog_renderer_get_ring_wedge_bpath (double cx, double cy,
 			path[n_segs + 2].code = ART_LINETO;
 			path[n_segs + 3].code = ART_END;
 		}
-	} else 
+	} else if (fill && ellipse) {
+		if (draw_in) {
+			path[n_segs + 1].x3 = cx + rx_in * cos (th0);
+			path[n_segs + 1].y3 = cy + ry_in * sin (th0);
+			path[n_segs + 1].code = ART_MOVETO;
+			path[2 * n_segs + 2].code = ART_END;
+		} else
+			path[n_segs + 1].code = ART_END;
+	} else
 		/* Arc */
 		path[n_segs + 1].code = ART_END;
 
@@ -385,12 +407,12 @@ gog_renderer_get_ring_wedge_bpath (double cx, double cy,
 		path[i].code = ART_CURVETO;
 		th_out -= th_delta;
 		if (draw_in) {
-			path[i+n_segs+1].x1 = cx + rx_in * (cos (th_in) - t * sin (th_in));
-			path[i+n_segs+1].y1 = cy + ry_in * (sin (th_in) + t * cos (th_in));
+			path[i+n_segs+1].x1 = cx + rx_in * (cos (th_in) + t * sin (th_in));
+			path[i+n_segs+1].y1 = cy + ry_in * (sin (th_in) - t * cos (th_in));
 			path[i+n_segs+1].x3 = cx + rx_in * cos (th_in + th_delta);
 			path[i+n_segs+1].y3 = cy + ry_in * sin (th_in + th_delta);
-			path[i+n_segs+1].x2 = path[i+n_segs+1].x3 + rx_in * t * sin (th_in + th_delta);
-			path[i+n_segs+1].y2 = path[i+n_segs+1].y3 - ry_in * t * cos (th_in + th_delta);
+			path[i+n_segs+1].x2 = path[i+n_segs+1].x3 - rx_in * t * sin (th_in + th_delta);
+			path[i+n_segs+1].y2 = path[i+n_segs+1].y3 + ry_in * t * cos (th_in + th_delta);
 			path[i+n_segs+1].code = ART_CURVETO;
 			th_in += th_delta;
 		}
@@ -428,7 +450,7 @@ gog_renderer_draw_ring_wedge (GogRenderer *rend, double cx, double cy,
 	if (path == NULL)
 		return;
 
-	if (rx_in >= 0.0 && ry_in >= 0.0)
+	if (go_add_epsilon (rx_in) >= 0.0 && go_add_epsilon (ry_in) >= 0.0)
 		(klass->draw_bezier_polygon) (rend, path, narrow);
 	else
 		(klass->draw_bezier_path) (rend, path);
@@ -854,16 +876,7 @@ gog_renderer_push_selection_style (GogRenderer *renderer)
 GogRenderer*
 gog_renderer_new_for_pixbuf (GogGraph *graph)
 {
-#ifdef GOFFICE_WITH_CAIRO
 	return g_object_new (GOG_RENDERER_CAIRO_TYPE, "model", graph, NULL);
-#else
-#ifdef GOFFICE_WITH_GTK
-	return g_object_new (GOG_RENDERER_PIXBUF_TYPE, "model", graph, NULL);
-#else
-	g_warning ("No cairo AND no gtk ???");
-	return NULL;
-#endif
-#endif
 }
 
 /**
@@ -878,18 +891,8 @@ gog_renderer_new_for_pixbuf (GogGraph *graph)
 gboolean
 gog_renderer_update (GogRenderer *renderer, double w, double h, double zoom)
 {
-#ifdef GOFFICE_WITH_CAIRO
 	g_return_val_if_fail (IS_GOG_RENDERER_CAIRO (renderer), FALSE);
 	return gog_renderer_cairo_update (GOG_RENDERER_CAIRO (renderer), w, h, zoom);
-#else
-#ifdef GOFFICE_WITH_GTK
-	g_return_val_if_fail (IS_GOG_RENDERER_PIXBUF (renderer), FALSE);
-	return gog_renderer_pixbuf_update (GOG_RENDERER_PIXBUF (renderer), w, h, zoom);
-#else
-	g_warning ("No cairo AND no gtk ???");
-	return FALSE;
-#endif
-#endif
 }
 
 /** 
@@ -901,18 +904,8 @@ gog_renderer_update (GogRenderer *renderer, double w, double h, double zoom)
 GdkPixbuf*
 gog_renderer_get_pixbuf (GogRenderer *renderer)
 {
-#ifdef GOFFICE_WITH_CAIRO
 	g_return_val_if_fail (IS_GOG_RENDERER_CAIRO (renderer), NULL);
      	return gog_renderer_cairo_get_pixbuf (GOG_RENDERER_CAIRO (renderer));
-#else
-#ifdef GOFFICE_WITH_GTK
-	g_return_val_if_fail (IS_GOG_RENDERER_PIXBUF (renderer), NULL);
-	return gog_renderer_pixbuf_get (GOG_RENDERER_PIXBUF (renderer));
-#else
-	g_warning ("No cairo AND no gtk ???");
-	return NULL;
-#endif
-#endif
 }
 
 /**
@@ -931,18 +924,6 @@ gog_renderer_new_for_format (GogGraph *graph, GOImageFormat format)
 	GType type = G_TYPE_INVALID;
 	
 	switch (format) {
-		case GO_IMAGE_FORMAT_PNG:
-		case GO_IMAGE_FORMAT_JPG:
-#ifdef GOFFICE_WITH_CAIRO
-			type = GOG_RENDERER_CAIRO_TYPE;
-#else
-#ifdef GOFFICE_WITH_GTK
-			type = GOG_RENDERER_PIXBUF_TYPE;
-			g_warning ("No cairo AND no gtk ???");
-			return NULL;
-#endif
-#endif
-			break;
 		case GO_IMAGE_FORMAT_SVG:
 #ifdef GOG_RENDERER_CAIRO_WITH_SVG
 			type = GOG_RENDERER_CAIRO_TYPE;
@@ -960,7 +941,10 @@ gog_renderer_new_for_format (GogGraph *graph, GOImageFormat format)
 			type = GOG_RENDERER_CAIRO_TYPE;
 #endif
 			break;
+		case GO_IMAGE_FORMAT_PNG:
+		case GO_IMAGE_FORMAT_JPG:
 		default:
+			type = GOG_RENDERER_CAIRO_TYPE;
 			break;
 	}
 	
@@ -992,7 +976,11 @@ gog_renderer_export_image (GogRenderer *renderer, GOImageFormat format,
 	GogRendererClass *klass;
 
 	g_return_val_if_fail (IS_GOG_RENDERER (renderer), FALSE);
-       
+
+	if (x_dpi <= 0.)
+		x_dpi = 96.;
+	if (y_dpi <= 0.)
+		y_dpi = 96.;
 	gog_graph_force_update (renderer->model);
 
 	klass = GOG_RENDERER_GET_CLASS (renderer);
