@@ -7,11 +7,161 @@
 
 #include <goffice/goffice-config.h>
 #include "regutf8.h"
+#include <pcre.h>
 #include "go-glib-extras.h"
 #include <gsf/gsf-impl-utils.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <string.h>
 
+/* ------------------------------------------------------------------------- */
+
+void
+go_regfree (GORegexp *gor)
+{
+  if (gor->ppcre) {
+    pcre_free (gor->ppcre);
+    gor->ppcre = NULL;
+  }
+}
+
+size_t
+go_regerror (int errcode, const GORegexp *gor, char *dst, size_t dstsize)
+{
+  const char *err;
+  size_t errlen;
+
+  switch (errcode) {
+  case REG_NOERROR: err = "?"; break;
+  case REG_NOMATCH: err = _("Pattern not found."); break;
+  default:
+  case REG_BADPAT: err = _("Invalid pattern."); break;
+  case REG_ECOLLATE: err = _("Invalid collating element."); break;
+  case REG_ECTYPE: err = _("Invalid character class name."); break;
+  case REG_EESCAPE: err = _("Trailing backslash."); break;
+  case REG_ESUBREG: err = _("Invalid back reference."); break;
+  case REG_EBRACK: err = _("Unmatched left bracket."); break;
+  case REG_EPAREN: err = _("Parenthesis imbalance."); break;
+  case REG_EBRACE: err = _("Unmatched \\{."); break;
+  case REG_BADBR: err = _("Invalid contents of \\{\\}."); break;
+  case REG_ERANGE: err = _("Invalid range end."); break;
+  case REG_ESPACE: err = _("Out of memory."); break;
+  case REG_BADRPT: err = _("Invalid repetition operator."); break;
+  case REG_EEND: err = _("Premature end of pattern."); break;
+  case REG_ESIZE: err = _("Pattern is too big."); break;
+  case REG_ERPAREN: err = _("Unmatched ) or \\)"); break;
+  };
+
+  errlen = strlen (err);
+  if (dstsize > 0) {
+    size_t copylen = MIN (errlen, dstsize - 1);
+    memcpy (dst, err, copylen);
+    dst[copylen] = 0;
+  }
+
+  return errlen;
+}
+
+int
+go_regcomp (GORegexp *gor, const char *pat , int cflags)
+{
+  const char *errorptr;
+  int errorofs, errorcode;
+  pcre *r;
+  int coptions =
+    PCRE_UTF8 |
+    PCRE_NO_UTF8_CHECK |
+    ((cflags & REG_ICASE) ? PCRE_CASELESS : 0) |
+    ((cflags & REG_NEWLINE) ? PCRE_MULTILINE : 0);
+
+  gor->ppcre = r = pcre_compile2 (pat, coptions,
+				  &errorcode, &errorptr, &errorofs,
+				  NULL);
+  if (r == NULL) {
+    switch (errorcode) {
+    case 1: case 2: case 3: case 37: return REG_EESCAPE;
+    case 4: case 5: return REG_EBRACE;
+    case 6: return REG_EBRACK;
+    case 7: case 30: return REG_ECTYPE;
+    case 8: return REG_ERANGE;
+    case 9: case 10: return REG_BADRPT;
+    case 14: case 18: case 22: return REG_EPAREN;
+    case 15: return REG_ESUBREG;
+    case 19: case 20: return REG_ESIZE;
+    case 21: return REG_ESPACE;
+    default: return REG_BADPAT;
+    }
+  } else {
+    gor->re_nsub = pcre_info (r, NULL, NULL);
+    gor->nosub = (cflags & REG_NOSUB) != 0;
+    return 0;
+  }
+}
+
+int
+go_regexec (const GORegexp *gor, const char *txt,
+	    size_t nmatch, GORegmatch *pmatch, int eflags)
+{
+  size_t txtlen = strlen (txt);
+  int eoptions =
+    ((eflags & REG_NOTBOL) ? PCRE_NOTBOL : 0) |
+    ((eflags & REG_NOTEOL) ? PCRE_NOTEOL : 0);
+  int res;
+  int *offsets, *allocated;
+  int offsetcount;
+
+  /* We need to totally ignore nmatch and pmatch in this case.  */
+  if (gor->nosub)
+    nmatch = 0;
+
+  if (nmatch > 0) {
+    /* Paranoia.  */
+    if (nmatch >= G_MAXINT / sizeof (int) / 3)
+      return REG_ESPACE;
+
+    offsetcount = nmatch * 3;
+    /* FIXME: we really want g_try_new but that is recent.  */
+    offsets = allocated = g_new (int, offsetcount);
+    if (!offsets)
+      return REG_ESPACE;
+  } else {
+    offsets = allocated = NULL;
+    offsetcount = 0;
+  }
+
+  res = pcre_exec (gor->ppcre, NULL, txt, txtlen, 0, eoptions,
+		   offsets, offsetcount);
+  if (res >= 0) {
+    int i;
+
+    if (res == 0) res = nmatch;
+
+    for (i = 0; i < res; i++) {
+      pmatch[i].rm_so = offsets[i * 2];
+      pmatch[i].rm_eo = offsets[i * 2 + 1];
+    }
+    for (; i < (int)nmatch; i++) {
+      pmatch[i].rm_so = -1;
+      pmatch[i].rm_eo = -1;
+    }
+    g_free (allocated);
+    return REG_NOERROR;
+  }
+
+  g_free (allocated);
+  switch (res) {
+  case PCRE_ERROR_NOMATCH:
+    return REG_NOMATCH;
+  case PCRE_ERROR_BADUTF8:
+  case PCRE_ERROR_BADUTF8_OFFSET:
+    /* POSIX doesn't seem to foresee this kind of error.  */
+    return REG_BADPAT;
+  default:
+    return REG_ESPACE;
+  }
+}
+
+/* ------------------------------------------------------------------------- */
+
 static GObjectClass *parent_class;
 
 enum {
@@ -58,7 +208,7 @@ go_search_replace_compile (GoSearchReplace *sr)
 	int flags = 0;
 	int res;
 
-	g_return_val_if_fail (sr && sr->search_text, REG_EMPTY);
+	g_return_val_if_fail (sr && sr->search_text, REG_EEND);
 
 	kill_compiled (sr);
 
@@ -232,7 +382,7 @@ go_regexp_quote (GString *target, const char *s)
 /* ------------------------------------------------------------------------- */
 
 static gboolean
-match_is_word (const char *src, const regmatch_t *pm, gboolean bolp)
+match_is_word (const char *src, const GORegmatch *pm, gboolean bolp)
 {
 	/* The empty string is not a word.  */
 	if (pm->rm_so == pm->rm_eo)
@@ -298,7 +448,7 @@ inspect_case (const char *p, const char *pend)
 
 
 static char *
-calculate_replacement (GoSearchReplace *sr, const char *src, const regmatch_t *pm)
+calculate_replacement (GoSearchReplace *sr, const char *src, const GORegmatch *pm)
 {
 	char *res;
 
@@ -394,7 +544,7 @@ go_search_match_string (GoSearchReplace *sr, const char *src)
 	}
 
 	while (1) {
-		regmatch_t match;
+		GORegmatch match;
 		int ret = go_regexec (sr->comp_search, src, 1, &match, flags);
 
 		switch (ret) {
@@ -432,7 +582,7 @@ char *
 go_search_replace_string (GoSearchReplace *sr, const char *src)
 {
 	int nmatch;
-	regmatch_t *pmatch;
+	GORegmatch *pmatch;
 	GString *res = NULL;
 	int ret;
 	int flags = 0;
@@ -446,7 +596,7 @@ go_search_replace_string (GoSearchReplace *sr, const char *src)
 	}
 
 	nmatch = 1 + sr->comp_search->re_nsub;
-	pmatch = g_new (regmatch_t, nmatch);
+	pmatch = g_new (GORegmatch, nmatch);
 
 	while ((ret = go_regexec (sr->comp_search, src, nmatch, pmatch, flags)) == 0) {
 		if (!res) {
