@@ -539,7 +539,7 @@ gog_series1_5d_get_property (GObject *obj, guint param_id,
 	}
 }
 
-#ifdef WITH_GTK
+#ifdef GOFFICE_WITH_GTK
 #include <gtk/gtklabel.h>
 static void 
 gog_series1_5d_populate_editor (GogObject *obj,
@@ -614,7 +614,7 @@ gog_series1_5d_class_init (GogObjectClass *obj_klass)
 	gobject_klass->get_property = gog_series1_5d_get_property;
 	gobject_klass->finalize 	      = gog_series1_5d_finalize;
 	obj_klass->update 	      = gog_series1_5d_update;
-#ifdef WITH_GTK
+#ifdef GOFFICE_WITH_GTK
 	obj_klass->populate_editor    = gog_series1_5d_populate_editor;
 #endif
 	gog_series_klass->dim_changed = gog_series1_5d_dim_changed;
